#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import argparse

from os import environ
from traceback import format_exc


# ----------------------------------------------------------------------
def banner(version):
    """Comment"""

    return ('''
///////////////////////////////////////////////////////
// ..................................DMI...
// .............................:MMMM......
// .........................$MMMMM:........
// .........M.....,M,=NMMMMMMMMD...........
// ........MMN...MMMMMMMMMMMM,.............
// .......MMMMMMMMMMMMMMMMM~...............
// .......MMMMMMMMMMMMMMM..................
// ....?MMMMMMMMMMMMMMMN$I.................
// .?.MMMMMMMMMMMMMMMMMMMMMM...............
// .MMMMMMMMMMMMMMN........................
// 7MMMMMMMMMMMMMON$.......................
// ZMMMMMMMMMMMMMMMMMM.......libs.......
// .:MMMMMMMZ~7MMMMMMMMMO..................
// ....~+:.................................
//
// Plecost - Wordpress finger printer Tool - v%s
//
// Developed by:
//        Francisco Jesus Gomez aka ffranz | @ffranz - ffranz-[at]-iniqua.com
//        Daniel Garcia aka cr0hn | @ggdaniel - cr0hn-[at]-cr0hn.com
//
// Info: http://iniqua.com/labs/
// Repo: https://github.com/iniqua/plecost
// Bug report: libs@iniqua.com or https://github.com/iniqua/plecost/issues/
''') % version


# ----------------------------------------------------------------------
def main():

    from .api import run, __version__
    from .libs.data import PlecostOptions, PlecostDatabaseQuery
    from .libs.utils import log
    from .libs.db import db_query

    examples = '''
Examples:

    * Scan target using default 50 most common plugins:
        plecost TARGET
    * Update plugin list
        plecost --update-plugins
    * Update vulnerability database:
        plecost --update-cve
    * List available word lists:
        plecost -l
    * Use embedded 1000 most common word list:
        plecost -w plugin_list_1000.txt TARGET
        or: plecost -w plugin_list_1000 TARGET
    * Scan, using 10 concurrent network connections:
        plecost -w plugin_list_1000.txt --concurrency 10 TARGET
    * Scan using verbose mode and generate xml report:
        plecost -w plugin_list_1000.txt --concurrency 10 -o report.xml TARGET
    * Scan using verbose mode and generate json report:
        plecost -vvv --concurrency 10 -o report.json TARGET
    * Not show banner, and only test wordpress connectivity, without plugin or wordpress testing:
        plecost -v -np -nc -nv TARGET
    * Update CVE database:
        plecost --update-cve
    * Update plugins list:
        plecost --update-plugins
    * List plugins with associated vulnerabilities in local database:
        plecost --show-plugins
    '''

    parser = argparse.ArgumentParser(description='Plecost: Wordpress finger printer tool', epilog=examples,
                                     formatter_class=argparse.RawTextHelpFormatter)

    # Main options
    parser.add_argument("target", metavar="TARGET", nargs="*")
    parser.add_argument("-v", "--verbosity", dest="verbose", action="count", help="verbosity level: -v, -vv, -vvv.", default=0)
    parser.add_argument('-o', dest="OUTPUT_FILE", help="report file with extension: xml|json", default=None)

    # Scanner options
    gr_wordlist = parser.add_argument_group("scanner options")
    gr_wordlist.add_argument('-np', '--no-plugins', dest="NO_PLUGINS_VERSIONS", action="store_true", default=False,
                             help="do not try to find plugins versions")
    gr_wordlist.add_argument('-nc', '--no-check-wordpress', dest="NO_CHECK_WORDPRESS", action="store_true",
                             default=False, help="do not check Wordpress connectivity")
    gr_wordlist.add_argument('-nv', '--no-wordpress-version', dest="NO_CHECK_WORDPRESS_VERSION", action="store_true",
                             default=False, help="do not check Wordpress version")
    gr_wordlist.add_argument('-f', '--force-scan', dest="FORCE_SCAN", action="store_true",
                             default=False, help="force to scan even although not wordpress installation detected")
    gr_wordlist.add_argument('-j', '--jackass-modes', dest="JACKASS", action="store_true",
                             default=False, help="jackass mode: unlimited connections to remote host")

    # Wordlist
    gr_wordlist = parser.add_argument_group("wordlist options")
    gr_wordlist.add_argument('-w', '--wordlist', dest="WORDLIST", help="set custom word list. Default 200 most common",
                             default=None)
    gr_wordlist.add_argument('-l', '--list-wordlist', dest="LIST_WORDLIST", help="list embedded available word list",
                             action="store_true", default=False)

    # Performance options
    gr_performance = parser.add_argument_group("advanced options")
    gr_performance.add_argument('-c', '--concurrency', dest="CONCURRENCY", type=int, help="number of parallel processes.",
                                default=4)
    # gr_performance.add_argument('--proxy', dest="PROXY", help="proxy as format proxy:port.", default=None)
    gr_performance.add_argument('-nb', dest="NO_BANNER", action="store_true", help="don't display banner",
                                default=False)

    # Updater
    gr_update = parser.add_argument_group("update options")
    #gr_update.add_argument('--update-core', dest="UPDATE_CORE", action="store_true", help="Update Plecost core.", default=False)
    gr_update.add_argument('--update-cve', dest="UPDATE_CVE", action="store_true", help="Update CVE database.", default=False)
    gr_update.add_argument('--update-plugins', dest="UPDATE_PLUGINS", action="store_true",
                           help="Update plugins.", default=False)
    gr_update.add_argument('--update-all', dest="UPDATE_ALL", action="store_true", help="Update CVE, plugins, and core.", default=False)

    # Database query
    gr_query = parser.add_argument_group("database search")
    gr_query.add_argument("-sp", "--show-plugins", dest="show_plugin_list", action="store_true",
                          help="display plugins in database")
    gr_query.add_argument("-vp", "--plugin-cves", dest="show_plugin_cves", help="display CVEs for plugin")
    gr_query.add_argument("--cve", dest="cve_details", help="display details of CVE")


    args = parser.parse_args()

    # Diplay banner
    if args.NO_BANNER is True:
        print("\n// Plecost - Wordpress finger printer Tool - v%s\n" % __version__)
    else:
        print(banner(__version__))

    # Set log function
    environ["PLECOST_LOG_LEVEL"] = str(args.verbose)

    # Update self
    # if args.UPDATE_CORE:
    #     from libs.updaters import update_core
    #     update_core(log)
    #     exit(0)

    # Update CVE
    if args.UPDATE_CVE:
        from .libs.updaters import update_cve
        update_cve(log)
        exit(0)

    # Update plugins
    if args.UPDATE_PLUGINS:
        from .libs.updaters import update_plugins
        update_plugins(log)
        exit(0)

    # Update all
    if args.UPDATE_ALL:
        from .libs.updaters import update_cve, update_plugins
        # update_core(log)
        update_cve(log)
        update_plugins(log)
        exit(0)

    # List available word lists
    if args.LIST_WORDLIST:
        from .libs.wordlist import list_wordlists
        log("Available word lists:\n")

        found = False
        for i, w in enumerate(list_wordlists(), 1):
            if not w.startswith("plugin"):
                continue
            found = True
            log("   %s - %s\n" % (i, w))
        if not found:
            log("   [!] No Word lists available\n")

        log("\n")
        exit(0)

    # --------------------------------------------------------------------------
    # Data base query
    # --------------------------------------------------------------------------
    if args.show_plugin_list or \
            args.show_plugin_cves or \
            args.cve_details:

        if args.show_plugin_cves:
            _action = "plugin_cves"
        elif args.show_plugin_list:
            _action = "plugin_list"
        else:
            _action = "cve"

        print(db_query(PlecostDatabaseQuery(action=_action,
                                            parameter=args.show_plugin_cves or args.cve_details)))

        print("[*] Done!\n")
        exit(0)

    # Targets selected?
    if not args.target:
        print("[!] You must specify a valid target. Type '-h' for help.\n")
        exit(0)

    try:
        # Set config
        config = PlecostOptions(target=args.target[0],
                                # proxy=args.PROXY,
                                concurrency=args.CONCURRENCY,
                                verbosity=args.verbose,
                                log_function=log,
                                report=args.OUTPUT_FILE,
                                wordlist=args.WORDLIST,
                                no_check_wordpress=args.NO_CHECK_WORDPRESS,
                                no_check_plugins=args.NO_PLUGINS_VERSIONS,
                                no_check_wordpress_version=args.NO_CHECK_WORDPRESS_VERSION,
                                force_scan=args.FORCE_SCAN)

        # Run Plecost
        run(config)
    except KeyboardInterrupt:
        log("[*] Exiting ...\n")
    except Exception as e:
        if args.verbose > 2:
            print(format_exc())
        print("\n[!] %s\n" % e)


if __name__ == "__main__" and __package__ is None:
    # --------------------------------------------------------------------------
    #
    # INTERNAL USE: DO NOT MODIFY THIS SECTION!!!!!
    #
    # --------------------------------------------------------------------------
    import sys
    import os
    parent_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    sys.path.insert(1, parent_dir)
    import plecost_lib
    __package__ = str("plecost_lib")
    del sys, os

    main()


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


__all__ = ["is_remote_a_wordpress", "get_wordpress_version"]

import re
import asyncio

from urllib.parse import urljoin

from ..db import DB
from ..wordlist import get_wordlist
from ..data import PlecostWordPressInfo
from ..utils import get_diff_ratio, update_progress, download, colorize


# ----------------------------------------------------------------------
# WordPress testing functions
# ----------------------------------------------------------------------
@asyncio.coroutine
def is_remote_a_wordpress(base_url, error_page, downloader):
    """
    This functions checks if remote host contains a WordPress installation.

    :param base_url: Base url
    :type base_url: basestring

    :param error_page: error page content
    :type error_page: basestring

    :param downloader: download function. This function must accept only one parameter: the URL
    :type downloader: function

    :return: True if target contains WordPress installation. False otherwise.
    :rtype: bool
    """
    total_urls = 0
    urls_found = 0

    for url in update_progress(get_wordlist("wordpress_detection.txt"), prefix_text="   "):
        total_urls += 1
        # Fix the url for urljoin
        path = url[1:] if url.startswith("/") else url

        headers, status, content = yield from downloader(urljoin(base_url, path))

        if status == 200:

            # Try to detect non-default error pages
            ratio = get_diff_ratio(content, error_page)
            if ratio < 0.35:
                urls_found += 1

    # If Oks > 85% continue
    if (urls_found / float(total_urls)) < 0.85:
        headers, status, content = yield from downloader(urljoin(base_url, "/wp-admin/"))
        if status == 302 and "wp-login.php?redirect_to=" in headers.get("location", ""):
            return True
        elif status == 301 and "/wp-admin/" in headers.get("location", ""):
            return True
        elif status == 200:
            return True
        else:
            return False
    else:
        return True


# ----------------------------------------------------------------------
@asyncio.coroutine
def get_wordpress_version(url, downloader, db):
    """
    This functions checks remote WordPress version.

    :param url: site to looking for WordPress version
    :type url: basestring

    :param downloader: download function. This function must accept only one parameter: the URL
    :type downloader: function

    :param db: cve database instance
    :type db: DB

    :return: PlecostWordPressInfo instance.
    :rtype: `PlecostWordPressInfo`
    """
    url_version = {
        # Generic
        "wp-login.php": r"(;ver=)([0-9\.]+)([\-a-z]*)",

        # For WordPress 3.8
        "wp-admin/css/wp-admin-rtl.css": r"(Version[\s]+)([0-9\.]+)",
        "wp-admin/css/wp-admin.css": r"(Version[\s]+)([0-9\.]+)"
    }

    # --------------------------------------------------------------------------
    #
    # Get installed version:
    #
    # --------------------------------------------------------------------------

    # --------------------------------------------------------------------------
    # Method 1: Looking for in readme.txt
    # --------------------------------------------------------------------------
    headers, status, curr_content = yield from downloader(urljoin(url, "/readme.html"))

    curr_ver = None
    if curr_content is not None:

        curr_ver = re.search(r"""(<br[\s]*/>[\s]*[Vv]ersion[\s]*)([\d]\.[\d]\.*[\d]*)""", curr_content)
        if curr_ver is None:
            curr_ver = None
        else:
            if len(curr_ver.groups()) != 2:
                curr_ver = None
            else:
                curr_ver = curr_ver.group(2)

    # --------------------------------------------------------------------------
    # Method 1: Looking for meta tag
    # --------------------------------------------------------------------------
    _, _, curr_content_2 = yield from downloader(url)

    # Try to find the info
    cur_ver_2 = None
    if curr_content_2 is not None:
        cur_ver_2 = re.search(r'''(<meta name=\"generator\" content=\"WordPress[\s]+)([0-9\.]+)''', curr_content_2)
        if cur_ver_2 is None:
            cur_ver_2 = None
        else:
            if len(cur_ver_2.groups()) != 2:
                cur_ver_2 = None
            else:
                cur_ver_2 = cur_ver_2.group(2)

    # --------------------------------------------------------------------------
    # Match versions of the different methods
    # --------------------------------------------------------------------------
    return_current_version = "unknown"
    if curr_ver is None and cur_ver_2 is None:
        return_current_version = "unknown"
    elif curr_ver is None and cur_ver_2 is not None:
        return_current_version = cur_ver_2
    elif curr_ver is not None and cur_ver_2 is None:
        return_current_version = curr_ver
    elif curr_ver is not None and cur_ver_2 is not None:
        if curr_ver != cur_ver_2:
            return_current_version = cur_ver_2
        else:
            return_current_version = curr_ver
    else:
        return_current_version = "unknown"

    # If Current version not found
    if return_current_version == "unknown":
        for url_pre, regex in url_version.items():
            # URL to find wordpress version
            url_current_version = urljoin(url, url_pre)
            _, _, current_version_content = yield from download(url_current_version, auto_redirect=False)

            # Find the version
            if current_version_content is not None:
                tmp_version = re.search(regex, current_version_content)

                if tmp_version is not None:
                    return_current_version = tmp_version.group(2)
                    break  # Found -> stop search

    # --------------------------------------------------------------------------
    # Get last version
    # --------------------------------------------------------------------------

    # URL to get last version of WordPress available
    _, _, last_version_content = yield from downloader("https://wordpress.org/download/")

    last_version = re.search("(WordPress\&nbsp\;)([0-9\.]*)", str(last_version_content))
    if last_version is None:
        last_version = "unknown"
    else:
        if len(last_version.groups()) != 2:
            last_version = "unknown"
        else:
            last_version = last_version.group(2)

    # Get wordpress vulnerabilities
    return PlecostWordPressInfo(current_version=return_current_version,
                                last_version=last_version,
                                vulnerabilities=get_wordpress_vulnerabilities(return_current_version, db))


# ----------------------------------------------------------------------
def get_wordpress_vulnerabilities(current_version, db):
    """
    Get CVEs associated to the installed Wordpress version

    :param current_version: PlecostWordPressInfo instance
    :type current_version: PlecostWordPressInfo

    :param db: cve database instance
    :type db: DB

    """
    if not isinstance(db, DB):
        raise TypeError("Expected DB, got '%s' instead" % type(db))

    if current_version is "unknown" or \
            not current_version:
        return []

    # Get CVE list
    return [x for x in db.query_wordpress(current_version)]


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#
# import re
# import csv
# import httplib
#
# from Queue import Queue
# from threading import Thread
# from functools import partial
# from datetime import datetime
# from urlparse import urlparse, urljoin
#
# from .data import *  # noqa
# from .wordlist import *  # noqa
# from .exceptions import *  # noqa
# from .utils import colorize, generate_error_page, download, get_diff_ratio, get_data_folder
#
#
# Probar a descargar el HTACCESS
# Ver si, con el usar el user agent de google el contenido es diferente -> malware (http://cantonbecker.com/work/musings/2009/how-to-search-for-backdoors-in-a-hacked-wordpress-site/)
#
#
# # ----------------------------------------------------------------------
# # Internal functions
# # ----------------------------------------------------------------------
# def _is_remote_a_wordpress(base_url, error_page, connection):
#     """
#     This functions checks if remote host contains a WordPress installation.
#
#     :param base_url: Base url
#     :type base_url: basestring
#
#     :param error_page: error page content
#     :type error_page: basestring
#
#     :param connection: HTTPConnection instance
#     :type connection: `HTTPConnection`
#
#     :return: True if target contains WordPress installation. False otherwise.
#     :rtype: bool
#     """
#     total_urls = 0
#     urls_found = 0
#
#     for url in get_wordlist("wordpress_detection.txt"):
#         total_urls += 1
#         # Fix the url for urljoin
#         path = url[1:] if url.startswith("/") else url
#         headers, status, content = download(urljoin(base_url, path), connection)
#
#         if status == 200:
#
#             # Try to detect non-default error pages
#             ratio = get_diff_ratio(content, error_page)
#             if ratio < 0.35:
#                 urls_found += 1
#
#     # If Oks > 85% continue
#     if (urls_found / float(total_urls)) < 0.85:
#         headers, status, content = download(urljoin(base_url, "/wp-admin/"), connection)
#         if status == 302 and "wp-login.php?redirect_to=" in headers.get("location", ""):
#             return True
#         elif status == 301 and "/wp-admin/" in headers.get("location", ""):
#             return True
#         elif status == 200:
#             return True
#         else:
#             return False
#     else:
#         return True
#
#
# # ----------------------------------------------------------------------
# def _get_wordpress_version(base_url, connection):
#     """
#     This functions checks remote WordPress version.
#
#     :param base_url: site to looking for WordPress version
#     :type base_url: basestring
#
#     :param connection: HTTPConnection instance
#     :type connection: `HTTPConnection`
#
#     :return: PlecostWordPressInfo instance.
#     :rtype: `PlecostWordPressInfo`
#     """
#     url_version = {
#         # Generic
#         "wp-login.php": r"(;ver=)([0-9\.]+)([\-a-z]*)",
#
#         # For WordPress 3.8
#         "wp-admin/css/wp-admin-rtl.css": r"(Version[\s]+)([0-9\.]+)",
#         "wp-admin/css/wp-admin.css": r"(Version[\s]+)([0-9\.]+)"
#     }
#
#     # --------------------------------------------------------------------------
#     #
#     # Get installed version:
#     #
#     # --------------------------------------------------------------------------
#
#     # --------------------------------------------------------------------------
#     # Method 1: Looking for in readme.txt
#     # --------------------------------------------------------------------------
#     curr_content = download(urljoin(base_url, "/readme.html"), connection)[2]
#
#     curr_ver = re.search(r"""(<br/>[\s]*[vV]ersion[\s]*)([0-9\.]*)""", curr_content)
#     if curr_ver is None:
#         curr_ver = None
#     else:
#         if len(curr_ver.groups()) != 2:
#             curr_ver = None
#         else:
#             curr_ver = curr_ver.group(2)
#
#     # --------------------------------------------------------------------------
#     # Method 1: Looking for meta tag
#     # --------------------------------------------------------------------------
#     curr_content_2 = download(base_url, connection)[2]
#
#     # Try to find the info
#     cur_ver_2 = re.search(r'''(<meta name=\"generator\" content=\"WordPress[\s]+)([0-9\.]+)''', curr_content_2)
#     if cur_ver_2 is None:
#         cur_ver_2 = None
#     else:
#         if len(cur_ver_2.groups()) != 2:
#             cur_ver_2 = None
#         else:
#             cur_ver_2 = cur_ver_2.group(2)
#
#     # --------------------------------------------------------------------------
#     # Match versions of the different methods
#     # --------------------------------------------------------------------------
#     return_current_version = "unknown"
#     if curr_ver is None and cur_ver_2 is None:
#         return_current_version = "unknown"
#     elif curr_ver is None and cur_ver_2 is not None:
#         return_current_version = cur_ver_2
#     elif curr_ver is not None and cur_ver_2 is None:
#         return_current_version = curr_ver
#     elif curr_ver is not None and cur_ver_2 is not None:
#         if curr_ver != cur_ver_2:
#             return_current_version = cur_ver_2
#         else:
#             return_current_version = curr_ver
#     else:
#         return_current_version = "unknown"
#
#     # If Current version not found
#     if return_current_version == "unknown":
#         for url_pre, regex in url_version.iteritems():
#             # URL to find wordpress version
#             url_current_version = urljoin(base_url, url_pre)
#             current_version_content = download(url_current_version, connection)[2]
#
#             # Find the version
#             tmp_version = re.search(regex, current_version_content)
#
#             if tmp_version is not None:
#                 return_current_version = tmp_version.group(2)
#                 break  # Found -> stop search
#
#     # --------------------------------------------------------------------------
#     # Get last version
#     # --------------------------------------------------------------------------
#     wordpress_connection = httplib.HTTPConnection("wordpress.org")
#
#     # URL to get last version of WordPress available
#     last_version_content = download("/download/", wordpress_connection)[2]
#
#     last_version = re.search("(WordPress&nbsp;)([0-9\.]*)", last_version_content)
#     if last_version is None:
#         last_version = "unknown"
#     else:
#         if len(last_version.groups()) != 2:
#             last_version = "unknown"
#         else:
#             last_version = last_version.group(2)
#
#     return PlecostWordPressInfo(current_version=return_current_version,
#                                 last_version=last_version)
#
#
# # ----------------------------------------------------------------------
# # Downloads functions
# # ----------------------------------------------------------------------
# def _download_plugin_simple(url_base, error_page, log, http_handler, data):
#     """
#     This functions download and URL, if pass callback function validation.
#
#     :param url_base: Base url where looking for plugins
#     :type url_base: basestring
#
#     :param error_page: Error page content as raw.
#     :type error_page: basestring
#
#     :param log: logging function, as format: log(message, level)
#     :type log: function
#
#     :param http_handler: HTTPConnection instance.
#     :type http_handler: `HTTPConnection`
#
#     :param data: list with plugin info. This list comes from csv iteration. Format:
#         data[0] => plugin uri
#         data[1] => plugin name
#         data[2] => plugin last version
#         data[3] => CVEs, separated by "|" character.
#     :type data: list
#
#     :return: PlecostPluginInfo instance
#     :rtype: PlecostPluginInfo|None
#     """
#
#     if data[0] == "quit":
#         return None
#
#     # Plugin properties
#     plugin_uri = data[0]
#     plugin_name = data[1]
#     plugin_last_version = data[2]
#     plugin_cves = [] if data[3] == "" else data[3].split("|")
#
#     # Test each URL with possible plugin version info
#     for target, regex in urls_plugin_regex.iteritems():
#
#         # Make Plugin url
#         partial_plugin_url = "%s/wp-content/plugins/%s/%s" % (url_base, data[0], target)
#
#         # Debug info
#         log("    |- Trying: %s (%s)...\n" % (colorize(data[0]), partial_plugin_url), 4)
#
#         # Download the info
#         headers, status, content = download(partial_plugin_url, http_handler)
#
#         # --------------------------------------------------------------------------
#         # Looking for plugin info
#         # --------------------------------------------------------------------------
#         plugin_installed_version = None
#         if status == 403:  # Installed, but inaccessible
#             plugin_installed_version = "Unknown"
#         elif status == 200:
#             # Check if page is and non-generic not found page with 404 code
#             if get_diff_ratio(error_page, content) < 0.52:
#                 # Find the version
#                 tmp_version = regex.search(content)
#
#                 if tmp_version is not None:
#                     plugin_installed_version = tmp_version.group(2)
#
#         # Store info
#         if plugin_installed_version is not None:
#             plugin = PlecostPluginInfo(current_version=plugin_installed_version,
#                                        last_version=plugin_last_version,
#                                        plugin_name=plugin_name,
#                                        plugin_uri=partial_plugin_url,
#                                        cves=plugin_cves)
#             text = (
#                 "    [%(symbol)s] Plugin found: %(name)s\n"
#                 "        |_Latest version: %(last)s\n"
#                 "        |_Installed version: %(curr)s\n"
#             ) % {
#                 "symbol": colorize("!", "red") if plugin.is_outdated else "i",
#                 "name": colorize(plugin.plugin_name, "blue"),
#                 "last": colorize(plugin.latest_version),
#                 "curr": colorize(plugin.current_version, "red") if plugin.is_outdated else plugin.current_version}
#
#             # Print
#             log(text, 0)
#
#             # Print CVE list
#             if plugin_cves:
#                 log("        |_CVE list:\n")
#                 for cve in plugin_cves:
#                     text = (
#                         "        |__%(cve)s: (http://cve.mitre.org/cgi-bin/cvename.cgi?name=%(cve)s)\n"
#                     ) % {"cve": colorize(cve, "red")}
#
#                     log(text, 0)
#
#             return plugin  # Plugin found -> not more URL test for this plugin
#
#     return None
#
#
# # ----------------------------------------------------------------------
# def _download_plugin_concurrent(url_base, error_page, log, in_queue, out_queue, http_handler):
#     """
#     Download an URL in a concurrent mode.
#
#     :param url_base: Base url where looking for plugins
#     :type url_base: basestring
#
#     :param error_page: Error page content as raw.
#     :type error_page: basestring
#
#     :param log: logging function, as format: log(message, level)
#     :type log: function
#
#     :param in_queue: input Queue witch receive the URLs.
#     :type in_queue: Queue
#
#     :param out_queue: output Queue witch function store the result info.
#     :type out_queue: Queue
#
#     :param http_handler: HTTPConnection instance.
#     :type http_handler: `HTTPConnection`
#     """
#     while True:
#         # Get the URL to download
#         data = in_queue.get()
#
#         # break?
#         if data[0] == "quit":
#             return
#
#         # Download
#         plugin = _download_plugin_simple(url_base, error_page, log, http_handler, data)
#
#         # Store result
#         if plugin:
#             out_queue.put(plugin, block=False)
#             out_queue.task_done()
#
#         # Set done
#         in_queue.task_done()
#
#
# # ----------------------------------------------------------------------
# def plugins_testing(host, error_page, log, data_list, concurrency=4):
#     """
#     Try to find plugins in remote host
#
#     :param host: Base host to test the URL list
#     :type host: str
#
#     :param data_list: list of urls to test
#     :type data_list: list
#
#     :param concurrency: max concurrency to process URLs
#     :type concurrency: int
#
#     :return: URLs of plugins and if pass check function or not. Format: [("url_to_plugin", True)]
#     :rtype: list((str, Bool))
#     """
#     if not isinstance(host, basestring):
#         raise TypeError("Expected basestring, got '%s' instead" % type(host))
#     if not isinstance(concurrency, int):
#         raise TypeError("Expected int, got '%s' instead" % type(concurrency))
#
#     schemes2port = dict(http=80,
#                         https=443)
#
#     # Get host name and port
#     parsed_url = urlparse(host)
#     host = parsed_url.netloc
#     url_base = parsed_url.path
#     port = schemes2port[parsed_url.scheme]
#     proxy = ""  # TODO
#
#     # If concurrency configured
#     if concurrency > 0:
#         q_in = Queue()
#         q_out = Queue()
#
#         # Concurrent method
#         for i in xrange(concurrency):
#             # Start HTTP persistent connections
#             if port == 80:  # Not SSL
#                 c = httplib.HTTPConnection(host, port)
#             else:
#                 c = httplib.HTTPSConnection(host, port)
#
#             t = Thread(target=_download_plugin_concurrent, kwargs={
#                 "url_base": url_base,
#                 "error_page": error_page,
#                 "log": log,
#                 "in_queue": q_in,
#                 "out_queue": q_out,
#                 "http_handler": c})
#             t.daemon = True
#             t.start()
#
#         add_data_function = partial(q_in.put, block=False)
#     else:
#         if port == 80:
#             c = httplib.HTTPConnection(host, port)
#         else:
#             c = httplib.HTTPSConnection(host, port)
#         add_data_function = partial(_download_plugin_simple, url_base, error_page, log, c)
#
#     results = []
#     results_append = results.append
#
#     # Process URLs
#     for url in data_list:
#         if not url:
#             continue
#         plugin = add_data_function(url)
#
#         if plugin:
#             results_append(plugin)
#
#     # If is a multiprocess, recover the results
#     if concurrency > 0:
#         # Wait for ending of queues
#         q_out.join()
#         q_in.join()
#
#         map(results_append, q_out.queue)
#
#     # Stop workers
#     add_data_function(["quit"])
#
#     return results
#
#
# # ----------------------------------------------------------------------
# # Main code of functions
# # ----------------------------------------------------------------------
# def find_versions(args):
#     """
#     Main function to run libs as version finder.
#
#     :param args: PlecostOptions object
#     :type args: `PlecostOptions`
#
#     :return: PlecostResults object.
#     :rtype: `PlecostResults`
#
#     :raises: PlecostTargetNotAvailable, PlecostNotWordPressFound
#     """
#     # --------------------------------------------------------------------------
#     # Common vars
#     # --------------------------------------------------------------------------
#     parsed_url = urlparse(args.target)
#     host = parsed_url.hostname
#     concurrency = args.concurrency
#     log = args.log_function
#     proxy = args.proxy
#     is_color = args.colorize
#     start_time = datetime.now()
#
#     # --------------------------------------------------------------------------
#     # Test availability of target
#     # --------------------------------------------------------------------------
#     try:
#         log("[*] Testing target connection... ", 0)
#
#         con = httplib.HTTPConnection(host)
#         download("/", con)
#
#         log(colorize("ok!\n"), 0)
#     except Exception, e:
#         raise PlecostTargetNotAvailable(e)
#
#     # Error page content.
#     headers, status, error_page = download(generate_error_page(host), con)
#
#     if 1==1:
#         # --------------------------------------------------------------------------
#         # Check if remote host is a WordPress
#         # --------------------------------------------------------------------------
#         log("[*] Testing for WordPress installation... ", 0)
#
#         if not _is_remote_a_wordpress("/", error_page, con):
#             raise PlecostNotWordPressFound("No WordPress installations found in '%s'." % host)
#
#         log(colorize("ok!\n"), 0)
#
#         # --------------------------------------------------------------------------
#         # Check WordPress version
#         # --------------------------------------------------------------------------
#         log("[*] Getting WordPress version... ", 0)
#
#         wordpress_version = _get_wordpress_version(host, con)
#         if wordpress_version:
#             log("%s (latest: %s)\n" %
#                 (
#                     colorize("%s" % wordpress_version.current_version, "red"),
#                     colorize("%s" % wordpress_version.latest_version)
#                 ), 0)
#         else:
#             log(colorize("Unknown!\n", "red"), 0)
#
#     # --------------------------------------------------------------------------
#     # Check the plugins
#     # --------------------------------------------------------------------------
#     # Read plugins file and remove \n and \r
#     plugins = []
#     plugins_append = plugins.append
#
#     with open(args.wordlist, "rU") as f:
#         for plugin in f:
#             plugins_append(plugin.replace("\n", "").replace("\r", ""))
#
#     # Prepare csv file
#     cve_info = csv.reader(plugins)
#
#     # Find plugins
#     log("[*] Looking for plugins (wordlist: %s) ... \n" % args.wordlist[args.wordlist.rfind("/") + 1:], 0)
#     plugins_info = plugins_testing(args.target, error_page, log, cve_info, concurrency)
#     log("[*] Done! \n", 0)
#
#     # Set finish time
#     end_time = datetime.now()
#
#     # --------------------------------------------------------------------------
#     # Clean up
#     # --------------------------------------------------------------------------
#     con.close()
#
#     # --------------------------------------------------------------------------
#     # Make results
#     # --------------------------------------------------------------------------
#     return PlecostResults(target=args.target,
#                           start_time=start_time,
#                           end_time=end_time,
#                           wordpress_info=wordpress_version,
#                           plugins=plugins_info)
#
#
# __all__ = ["find_versions", "_is_remote_a_wordpress"]


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


"""
This file contains data structures of Plecost.
"""


import re

from sys import stdout
from abc import ABCMeta
from datetime import datetime
from os.path import exists, join

from .utils import get_data_folder
from .wordlist import list_wordlists
from .exceptions import PlecostWordListNotFound


# --------------------------------------------------------------------------
# API caller structure
# --------------------------------------------------------------------------
class PlecostOptions(object):
    """Plecost runner options"""

    # ----------------------------------------------------------------------
    def __init__(self, **kwargs):
        """
        :param proxy: Proxy as format: {HOST:PORT}. Default None.
        :type proxy: dict()
        
        :param target: URL target.
        :type target: str

        :param concurrency: maximum number of simultaneous connections. Default 4.
        :type concurrency: int
        
        :param log_function: function to call for display log info. Function format: FUNC(Message::str, Level::int)
        :type log_function: function(str, int)
        
        :param verbosity: verbosity level
        :type verbosity: int
        
        :param report_filename: report filename 
        :type report_filename: str

        :param colorize: Colorize output
        :type colorize: bool

        :param wordlist: Word list path (or embedded) selected find plugins in target.
        :type wordlist: str
        """
        self.__proxy = kwargs.get("proxy", {})
        self.__target = kwargs.get("target", None)
        self.__verbosity = kwargs.get("verbosity", 0)
        self.__concurrency = kwargs.get("concurrency", 4)
        self.__report_filename = kwargs.get("report", None)
        self.__log_function = kwargs.get("log_function",
                                         lambda message, log_level=0, current_log_level=None: stdout.write(message))
        self.__colorize = kwargs.get("colorize", True)
        self.__wordlist = kwargs.get("wordlist", None)
        self.__no_check_wordpress = kwargs.get("no_check_wordpress", False)
        self.__no_check_plugins = kwargs.get("no_check_plugins", False)
        self.__no_check_wordpress_version = kwargs.get("no_check_wordpress_version", False)
        self.__force_scan = kwargs.get("force_scan", False)
        self.__jackass = kwargs.get("jackass", False)

        # Check types and default values
        if not isinstance(self.__target, str):
            raise TypeError("Expected basestring, got '%s' instead" % type(self.__target))
        if not isinstance(self.__concurrency, int):
            raise TypeError("Expected int, got '%s' instead" % type(self.__concurrency))
        if self.__log_function is None:
            raise TypeError("Expected function, got '%s' instead" % type(self.__log_function))
        if not isinstance(self.__verbosity, int):
            raise TypeError("Expected int, got '%s' instead" % type(self.__verbosity))

        # Fix target:
        if not self.__target.startswith("http"):
            self.__target = "http://%s" % self.__target

        # Check word list and fix word list path
        normal_wordlist = list_wordlists()
        matches = {w.replace(".txt", ""): w for w in normal_wordlist}
        if self.__wordlist is None:
            self.__wordlist = join(get_data_folder(), "plugin_list_50.txt")
        if self.__wordlist in normal_wordlist:
            self.__wordlist = join(get_data_folder(), self.__wordlist)
        elif self.__wordlist in matches:
            self.__wordlist = join(get_data_folder(), matches[self.__wordlist])
        elif not exists(self.__wordlist):
            raise PlecostWordListNotFound("Word list not found")

    # ----------------------------------------------------------------------
    @property
    def jackass(self):
        """
        :return: Jackass mode enabled?
        :rtype: bool
        """
        return self.__jackass

    # ----------------------------------------------------------------------
    @property
    def force_scan(self):
        """
        :return: force scan even if not wordpress detected
        :rtype: bool
        """
        return self.__force_scan

    # ----------------------------------------------------------------------
    @property
    def no_check_wordpress_version(self):
        """
        :return: Check wordpress version? False: Yes. True: No
        :rtype: bool
        """
        return self.__no_check_wordpress_version

    # ----------------------------------------------------------------------
    @property
    def no_check_plugins(self):
        """
        :return: boolean that indicates if plecost must test plugins or not. True: No test. False: test plugins.
        :rtype: bool
        """
        return self.__no_check_plugins

    # ----------------------------------------------------------------------
    @property
    def no_check_wordpress(self):
        """
        :return: No check if Wordpress is available
        :rtype: bool
        """
        return self.__no_check_wordpress

    # ----------------------------------------------------------------------
    @property
    def target(self):
        """
        :return: Target
        :rtype: basestring
        """
        return self.__target
    
    # ----------------------------------------------------------------------
    @property
    def proxy(self):
        """
        :return: Proxy as format {HOST:PORT}
        :rtype: dict
        """
        return self.__proxy 
    
    # ----------------------------------------------------------------------
    @property
    def concurrency(self):
        """
        :return: Maximum number of concurrent connections.
        :rtype: int
        """
        return self.__concurrency

    # ----------------------------------------------------------------------
    @property
    def log_function(self):
        """
        :return: function to call for display log info. Function format: FUNC(Message::str, Level::int)
        :rtype: function(str, int)
        """
        return self.__log_function

    # ----------------------------------------------------------------------
    @property
    def verbosity(self):
        """
        :return: verbosity level.
        :rtype: int
        """
        return self.__verbosity
    
    # ----------------------------------------------------------------------
    @property
    def report_filename(self):
        """
        :return: Report filename
        :rtype: basestring
        """
        return self.__report_filename
    
    # ----------------------------------------------------------------------
    @property
    def colorize(self):
        """
        :return: 
        :rtype: 
        """
        return self.__colorize
    
    # ----------------------------------------------------------------------
    @property
    def wordlist(self):
        """
        :return: path or name (if embedded) wordlist.
        :rtype: basestring
        """
        return self.__wordlist


# --------------------------------------------------------------------------
# Results data structures
# --------------------------------------------------------------------------
class _PlecostBase(object, metaclass=ABCMeta):
    """Abstract class for all Plecost types"""

    # ----------------------------------------------------------------------
    def __init__(self, ver1, ver2):
        """
        :param ver1: current version of software to compare to.
        :type ver1: basestring
        
        :param ver2: latest version of software to compare to.
        :type ver2: basestring
        """
        if ver1 is None or ver1 is None:
            self._outdated = False
        else:
            if not isinstance(ver1, str):
                raise TypeError("Expected basestring, got '%s' instead" % type(ver1))
            if not isinstance(ver2, str):
                raise TypeError("Expected basestring, got '%s' instead" % type(ver2))

            # Is outdated?
            if self.__version_cmp(ver1, ver2) == -1:
                self._outdated = True
            else:
                self._outdated = False

    # ----------------------------------------------------------------------
    def __version_cmp(self, version1, version2):
        """
        Compare two software versions.

        :param version1: string with version number.
        :type version1: str

        :param version2: string with version number.
        :type version2: str

        :return: 1 if version 1 is greater. -1 if version2 if greater.
        :rtype: int
        """
        try:
            tup = lambda x: [int(y) for y in (x+'.0.0.0.0').split('.')][:4]
        except TypeError:
            return -1

        if version1.lower() == "trunk":
            return 1
        elif version2.lower() == "trunk":
            return -1
        else:
            if not re.search("([\\d]\\.[\\d]\\.*[\\d]*)", version1) or \
                    not re.search("([\\d]\\.[\\d]\\.*[\\d]*)", version2):
                return -1
            else:
                # return cmp(tup(version1), tup(version2))
                if tup(version1) > tup(version2):
                    return 1
                elif tup(version1) < tup(version2):
                    return -1
                else:
                    return 0

    # ----------------------------------------------------------------------
    @property
    def is_outdated(self):
        """
        :return: True if that software is outdated. None if unknown.
        :rtype: bool|None.
        """
        return self._outdated


# ----------------------------------------------------------------------
class PlecostWordPressInfo(_PlecostBase):
    """WordPress installation information"""

    # ----------------------------------------------------------------------
    def __init__(self, **kwargs):
        """
        :param current_version: Current version number as format: x.x.x.
        :type current_version: str

        :param last_version: Last version availble as format: x.x.x
        :type last_version: str
        """
        self.__current_version = kwargs.get("current_version", None)
        self.__last_version_available = kwargs.get("last_version", None)
        self.__vulnerabilities = kwargs.get("vulnerabilities", None)

        if not isinstance(self.__current_version, str):
            raise TypeError("Expected basestring, got '%s' instead" % type(self.__current_version))
        if not isinstance(self.__last_version_available, str):
            raise TypeError("Expected basestring, got '%s' instead" % type(self.__last_version_available))

        super(PlecostWordPressInfo, self).__init__(self.__current_version, self.__last_version_available)

    # ----------------------------------------------------------------------
    @property
    def vulnerabilities(self):
        """
        :return: list of string with cves
        :rtype: list(str)
        """
        return self.__vulnerabilities

    # ----------------------------------------------------------------------
    @property
    def latest_version(self):
        """
        :return: Last version of WordPress available as format: x.x.x
        :rtype: basestring
        """
        return self.__last_version_available
    
    # ----------------------------------------------------------------------
    @property
    def current_version(self):
        """
        :return: Current version installed in target as format: x.x.x
        :rtype: basestring
        """
        return self.__current_version


# --------------------------------------------------------------------------
class PlecostPluginInfo(_PlecostBase):
    """Plugin information in remote host"""

    # ----------------------------------------------------------------------
    def __init__(self, **kwargs):
        """
        :param current_version: Current version number as format: x.x.x.
        :type current_version: str

        :param last_version: Last version available as format: x.x.x
        :type last_version: str
        
        :param plugin_name: Plugin long name description. 
        :type plugin_name: basestring
        
        :param plugin_uri: Plugin URI 
        :type plugin_uri: basestring
        
        :param cves: list with CVEs related 
        :type cves: list(str)
        
        :param exploits: list with url to related exploits.
        :type exploits: list(str)
        """
        self.__plugin_name = kwargs.get("plugin_name", None)
        self.__plugin_uri = kwargs.get("plugin_uri", None)
        self.__current_version = kwargs.get("current_version", None)
        self.__last_version = kwargs.get("last_version", None)
        self.__cves = kwargs.get("cves", [])
        self.__exploits = kwargs.get("exploits", [])

        if not isinstance(self.__plugin_uri, str):
            raise TypeError("Expected basestring, got '%s' instead" % type(self.__plugin_uri))

        super(PlecostPluginInfo, self).__init__(self.__current_version, self.__last_version)

    # ----------------------------------------------------------------------
    @property
    def plugin_name(self):
        """
        :return: Plugin name.
        :rtype: basestring
        """
        return self.__plugin_name if self.__plugin_name else self.__plugin_uri

    # ----------------------------------------------------------------------
    @property
    def plugin_uri(self):
        """
        :return: Plugin URI
        :rtype: basestring
        """
        return self.__plugin_uri   
    
    # ----------------------------------------------------------------------
    @property
    def current_version(self):
        """
        :return: current version installed as format: x.x.x
        :rtype: basestring|None
        """
        return self.__current_version

    # ----------------------------------------------------------------------
    @property
    def latest_version(self):
        """
        :return: Latest version installed, if available, as format: x.x.x.
        :rtype: basestring|None
        """
        return self.__last_version
    
    # ----------------------------------------------------------------------
    @property
    def cves(self):
        """
        :return: CVEs associated to this plugin.
        :rtype: list(str)
        """
        return self.__cves
    
    # ----------------------------------------------------------------------
    @property
    def exploits(self):
        """
        :return: Exploits available for this plugin
        :rtype: list(str)
        """
        return self.__exploits


# --------------------------------------------------------------------------
class PlecostResults(object):
    """Plecost results"""

    # ----------------------------------------------------------------------
    def __init__(self, **kwargs):
        """
        :param target: tested target
        :type target: basestring

        :param start_time: start time of testing
        :type start_time: datetime

        :param end_time: end time of testing
        :type end_time: datetime

        :param wordpress_info: remote wordpress information. 
        :type wordpress_info: `PlecostWordPressInfo`
        
        :param plugins: plugins available in remote target.
        :type plugins: `list(PlecostPluginInfo)
        """
        self.__target = kwargs.get("target", None)
        self.__start_time = kwargs.get("start_time", datetime.now())
        self.__end_time = kwargs.get("end_time", datetime.now())
        self.__wordpress_info = kwargs.get("wordpress_info", None)
        self.__plugins = kwargs.get("plugins", None)

        if not isinstance(self.__target, str):
            raise TypeError("Expected basestring, got '%s' instead" % type(self.__target))
        if not isinstance(self.__wordpress_info, PlecostWordPressInfo):
            raise TypeError("Expected PlecostWordPressInfo, got '%s' instead" % type(self.__wordpress_info))
        if not isinstance(self.__plugins, list):
            raise TypeError("Expected list, got '%s' instead" % type(self.__plugins))
        else:
            for plugin in self.__plugins:
                if not isinstance(plugin, PlecostPluginInfo):
                    raise TypeError("Expected PlecostPluginInfo, got '%s' instead" % type(plugin))

        # Filter outdated plugins
        self.__outdated_plugins = []
        for plugin in self.__plugins:
            if plugin.is_outdated is True:
                self.__outdated_plugins.append(plugin)

    # ----------------------------------------------------------------------
    # Properties
    # ----------------------------------------------------------------------
    @property
    def target(self):
        """
        :return: tested target
        :rtype: basestring
        """
        return self.__target

    # ----------------------------------------------------------------------
    @property
    def wordpress_info(self):
        """
        :return: WordPress information object
        :rtype: `PlecostWordPressInfo`
        """
        return self.__wordpress_info
        
    # ----------------------------------------------------------------------
    @property
    def plugins(self):
        """
        :return: installed plugins found in target.
        :rtype: `list(PlecostPluginInfo)`
        """
        return self.__plugins

    # ----------------------------------------------------------------------
    @property
    def start_time(self):
        """
        :return: start time of testing
        :rtype: datetime
        """
        return self.__start_time

    # ----------------------------------------------------------------------
    @property
    def end_time(self):
        """
        :return: end time of testing
        :rtype: datetime
        """
        return self.__end_time

    # ----------------------------------------------------------------------
    @property
    def outdated_plugins(self):
        """
        :return: Only outdated plugins
        :rtype: `list(PlecostPluginInfo)`
        """
        return self.__outdated_plugins


# --------------------------------------------------------------------------
class PlecostDatabaseQuery:
    """Query database info"""

    # ----------------------------------------------------------------------
    def __init__(self, **kwargs):
        """Constructor"""
        self.action = kwargs.get("action", None)
        self.parameter = kwargs.get("parameter", None)

        if not isinstance(self.action, str):
            raise TypeError("Expected str, got '%s' instead" % type(self.action))



__all__ = [x for x in dir() if x.startswith("Plecost")]


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


__all__ = ["plugins_testing"]

import re
import asyncio

from os.path import join
from functools import partial
from urllib.parse import urljoin

from ..data import PlecostPluginInfo
from ..utils import colorize, get_diff_ratio, ConcurrentDownloader, get_data_folder

exp = re.compile(r"([Ss]table tag:[\s]*)([\svV]*[0-9\.]+|trunk)")
exp_change_log = re.compile(r"([\=\-]\s*)([\d]+\.[\d]+\.*[\d]*\.*[\d]*\.*[\d]*\.*[\d]*)(\s*[\=\-])")


# ----------------------------------------------------------------------
def _url_generator(url_base, data):
    """
    This functions download and URL, if pass callback function validation.

    :param url_base: Base url where looking for plugins
    :type url_base: basestring

    :param data: list with plugin info. This list comes from csv iteration. Format:
        data[0] => plugin uri
        data[1] => plugin name
        data[2] => plugin last version
        data[3] => CVEs, separated by "|" character.
    :type data: list

    :return: list of URLs generated
    :rtype: list(str)
    """

    urls_plugin_regex = {
        "readme.txt": exp,
        "README.txt": exp,
        }

    results = []
    # Test each URL with possible plugin version info
    for target, regex in urls_plugin_regex.items():

        _path = "wp-content/plugins/%s/%s" % (data[0], target)

        # Make Plugin url
        results.append(urljoin(url_base, _path))

    return results


# ----------------------------------------------------------------------
def _plugin_analyze(data_map, error_page, db, log, url, headers, status, content):
    """
    This functions download and URL, if pass callback function validation.

    :param data_map: a dict with with plugin info, into a list. This list comes from csv iteration. Format:
        {
            plugin_name: [
                    [0] => plugin uri
                    [1] => plugin name
                    [2] => plugin last version
                    [3] => CVEs, separated by "|" character.
                ]
        }
    :type data_map: dict(str: list)

    :param error_page: Error page content as raw.
    :type error_page: basestring

    :param db: cve database instance
    :type db: DB
    
    :param log: logging function, as format: log(message, level)
    :type log: function

    :param url: current plugin URL to analyze
    :type url: str

    :param headers: dict with HTTP headers response
    :type headers: dict

    :param status: HTTP status response
    :type status: int

    :param content: Response of HTTP query
    :type content: str

    :return: PlecostPluginInfo instance
    :rtype: PlecostPluginInfo|None
    """
    if content is None:
        return None

    data = data_map[url]

    # Plugin properties
    plugin_uri = data[0]
    plugin_name = data[1]
    plugin_last_version = data[2]

    # --------------------------------------------------------------------------
    # Looking for plugin info
    # --------------------------------------------------------------------------
    plugin_installed_version = None
    if status == 403:  # Installed, but inaccessible
        plugin_installed_version = "Unknown"
    elif status == 200:
        # Check if page is and non-generic not found page with 404 code
        if get_diff_ratio(error_page, content) < 0.52:
            # Find the version
            tmp_version = exp.search(content)

            if tmp_version is not None:
                plugin_installed_version = tmp_version.group(2)

            # Try to improve version, looking for into changelog
            if plugin_installed_version is None or plugin_installed_version == "trunk":

                tmp_version_change_log = exp_change_log.search(content)

                if tmp_version_change_log is not None:
                    plugin_installed_version = tmp_version_change_log.group(2)

    # Store info
    if plugin_installed_version is not None:

        # --------------------------------------------------------------------------
        # Looking for CVE
        # --------------------------------------------------------------------------
        cves = db.query_plugin(plugin_uri.replace("_", "-"), plugin_installed_version)

        plugin = PlecostPluginInfo(current_version=plugin_installed_version,
                                   last_version=plugin_last_version,
                                   plugin_name=plugin_name,
                                   plugin_uri=url,
                                   cves=cves)
        text = ("\n    <%(symbol)s> Plugin found: %(name)s\n"
                "        |_Latest version: %(last)s\n"
                "        |_Installed version: %(curr)s"
               ) % {
                   "symbol": colorize("!", "red") if plugin.is_outdated else "i",
                   "name": colorize(plugin.plugin_name, "blue"),
                   "last": colorize(plugin.latest_version),
                   "curr": colorize(plugin.current_version, "red") if plugin.is_outdated else plugin.current_version}

        # Print
        log(text)

        # Print CVE list
        if plugin.cves:
            log("\n        |_CVE list:\n")
            for cve in plugin.cves:
                text = (
                           "        |__%(cve)s: (http://cve.mitre.org/cgi-bin/cvename.cgi?name=%(cve)s)\n"
                       ) % {"cve": colorize(cve, "red")}

                log(text)

        return plugin  # Plugin found -> not more URL test for this plugin

    else:
        return None


# ----------------------------------------------------------------------
@asyncio.coroutine
def plugins_testing(url, error_page, log, data_list, db, concurrency=4, loop=None, con=None):
    """
    Try to find plugins in remote url

    :param url: Base url to test the URL list
    :type url: str

    :param data_list: list of urls to test
    :type data_list: list

    :param db: cve database instance
    :type db: DB

    :param concurrency: max concurrency to process URLs
    :type concurrency: int

    :return: URLs of plugins and if pass check function or not. Format: [("url_to_plugin", True)]
    :rtype: list((str, Bool))
    """
    if not isinstance(url, str):
        raise TypeError("Expected basestring, got '%s' instead" % type(url))
    if not isinstance(concurrency, int):
        raise TypeError("Expected int, got '%s' instead" % type(concurrency))

    # Make URLs
    urls = {}
    for x in data_list:
        for u in _url_generator(url, x):
            urls[u] = x

    # Map function
    fn = partial(_plugin_analyze, urls, error_page, db, log)

    # Prepare concurrent connections
    cr = ConcurrentDownloader(fn, max_tasks=concurrency, loop=loop, connector=con, max_redirects=0)
    cr.add_url_list(urls)

    # Run and wait!
    yield from cr.run()

    return cr.results




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#



import re
import csv
import pickle

from time import sleep
from os.path import join
from codecs import decode
from random import randint
from chardet import detect
from bs4 import BeautifulSoup
from urllib.error import URLError
from urllib.request import urlopen

from ..utils import colorize, get_data_folder, update_progress


# --------------------------------------------------------------------------
def update_plugins(log):
    """
    Update data information.

    :param log: Log function as format: function(Message, Level)
    :type log: function(str, int)
    """
    # --------------------------------------------------------------------------
    # Config and vars
    # --------------------------------------------------------------------------
    regex_plugin_name = re.compile(r"(http[s]*://wordpress.org/plugins/)([a-zA-Z0-9\\\s_\-`!()\[\]{};:'.,<>?«»‘’]+)([/]*\">)([a-zA-Z0-9\\\s_\-`!()\[\]{};:'.,?«»‘’]*)",
                                   re.I)  # -> Group 2 and 3
    regex_plugin_version = re.compile(r"(Version</span>[\sa-zA-Z]*)([0-9\.]+)", re.I)  # -> Group 2
    wp_plugins_url = "http://wordpress.org/plugins/browse/popular/page/%s/"
    max_plugins = 1400

    file_out = join(get_data_folder(), "plugin_list_huge.txt")

    log("[*] Preparing for update plugins...\n")

    with open(file_out, "w") as out:

        already_processed = []
        already_processed_append = already_processed.append

        csv_file = csv.writer(out)

        total_plugins = 1

        # Looking for 85 * 12 (per page) = 1020  plugins
        for i in update_progress(range(1, 85), prefix_text="[*] Downloading plugins (slow): "):
            # 6 tries for each request
            for x in range(1, 6):
                try:
                    url = wp_plugins_url % i
                    wpage = urlopen(url).read()
                    break
                except URLError as e:
                    log("[%s] %s" % (
                        colorize("!", "red"),
                        colorize("Error while getting URL: %s. Attempt %s.\n" % (url, x))
                    ))
                    sleep(randint(1, 4))

                    # Maximum attempt reached
                    if x == 6:
                        log("[%s] %s" % (
                            colorize("!!", "red"),
                            colorize("Maximum time exceeded"),
                        ), 0)
                        return
                    else:
                        continue

            # Fix err
            try:
                wpage = decode(wpage, detect(wpage)["encoding"])
            except UnicodeEncodeError as e:
                log("[%s] Unicode error while processing url '%s'\n" % (
                    colorize("!", "red"),
                    colorize(url)
                ))

                log("    |- Error details: %s" % e, 3)
                continue

            # Parse
            bs = BeautifulSoup(wpage)

            # For each plugin
            for j, plugin_info in enumerate(bs.findAll("div", attrs={"class": "plugin-block"})):

                plugin_info = str(plugin_info)

                # --------------------------------------------------------------------------
                # Plugin name and URL
                # --------------------------------------------------------------------------
                plugin_n_t = regex_plugin_name.search(plugin_info)
                plugin_url = None
                plugin_name = None
                if plugin_n_t is None:
                    log("[%s] REGEX_PLUGIN_NAME can't found info for string: \n-------\n%s\n" %
                        (
                            colorize("!!!", "red"),
                            plugin_info
                        ), 2)
                else:
                    plugin_url = plugin_n_t.group(2) if len(plugin_n_t.groups()) >= 2 else None
                    plugin_name = plugin_n_t.group(4) if len(plugin_n_t.groups()) >= 4 else None

                # Coding fixes
                if plugin_name:
                    try:
                        # plugin_name = decode(plugin_name, detect(plugin_name.encode())["encoding"])
                        plugin_name = plugin_name
                    except UnicodeError:
                        try:
                            plugin_name = plugin_name.decode("UTF-8")
                        except UnicodeError:
                            plugin_name = plugin_url
                else:
                    plugin_name = plugin_url

                # --------------------------------------------------------------------------
                # Plugin version
                # --------------------------------------------------------------------------
                plugin_version = regex_plugin_version.search(plugin_info)
                if plugin_version is None:
                    log("[%s] REGEX_PLUGIN_VERSION can't found info for string: \n-------\n%s\n" %
                        (
                            colorize("!!!", "red"),
                            plugin_info
                        ), 2)
                else:
                    plugin_version = plugin_version.group(2)

                # Plugin is repeated and already processed?
                if plugin_url in already_processed:
                    log("[%s] Already processed plugin '%s'. Skipping\n" %
                        (
                            colorize("ii", "red"),
                            plugin_url
                        ), 2)
                    continue

                # --------------------------------------------------------------------------
                # We have all information to continue?
                # --------------------------------------------------------------------------
                if plugin_url is None or plugin_version is None:
                    log("[%s] Not enough information to store plugin for:\n%s\n" %
                        (
                            colorize("ii", "red"),
                            plugin_info
                        ), 2)
                    continue

                # Report status
                log("    |-- %s - Processing plugin: %s\n" %
                    (
                        colorize(total_plugins),
                        plugin_url
                    ), log_level=1)

                # Write to file
                try:
                    csv_file.writerow([plugin_url, plugin_name, plugin_version])
                except UnicodeEncodeError:
                    csv_file.writerow([plugin_url, plugin_url, plugin_version])

                # Save plugin
                already_processed_append(plugin_url)

                # Maximum number of plugins reached?
                total_plugins += 1

                if total_plugins >= max_plugins:
                    return

    # Creates splited files
    with open(file_out, "r") as all_plugins, \
            open(join(get_data_folder(), "plugin_list_10.txt"), 'w') as f_10, \
            open(join(get_data_folder(), "plugin_list_50.txt"), 'w') as f_50, \
            open(join(get_data_folder(), "plugin_list_100.txt"), 'w') as f_100, \
            open(join(get_data_folder(), "plugin_list_250.txt"), 'w') as f_250, \
            open(join(get_data_folder(), "plugin_list_1000.txt"), 'w') as f_1000:

        for i, line in enumerate(all_plugins.readlines(), start=1):

            _line = line

            if i < 11:
                f_10.write(_line)
            if i < 50:
                f_50.write(_line)
            if i < 100:
                f_100.write(_line)
            if i < 250:
                f_250.write(_line)
            if i < 1000:
                f_1000.write(_line)

    log("\n[*] Oks!\n")




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

from setuptools import setup, find_packages

files = ["resources/*"]

setup(
    name='plecost',
    version='1.0.1',
    packages=find_packages(),
    install_requires=["chardet", "termcolor", "BeautifulSoup4", "aiohttp"],
    url='https://github.com/iniqua/plecost/tree/python3',
    license='GPL2',
    author='Plecost team',
    author_email='libs@iniqua.com',
    package_data={'plecost_lib': files},
    entry_points={'console_scripts': [
        'plecost = plecost_lib.plecost:main',
        ]},
    description='Wordpress finger printer tool and vulnerabilities searcher',
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3',
        'Topic :: Security',
        ]
)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


"""
This file contains some orphan functions
"""
import urllib


__all__ = ["colorize", "generate_error_page", "get_diff_ratio", "get_data_folder", "check_redirects"]

import aiohttp
import asyncio

from urllib.parse import urljoin
from random import choice, randint
from difflib import SequenceMatcher
from string import ascii_letters, digits
from os.path import abspath, basename, dirname, join

try:
    from termcolor import colored
except ImportError:
    def colored(text, color):
        return text


# ----------------------------------------------------------------------
def log(message, log_level=0, current_log_level=None):
    """
    Auxiliary function to use as log level

    :param current_log_level: Log level selected at the moment of running the program
    :type current_log_level: int

    :param message: Message to display
    :type message: basestring

    :param log_level: log level: 0-4
    :type log_level: int
    """
    from sys import stdout
    from os import environ

    if current_log_level is None:
        try:
            _current_log_level = int(environ["PLECOST_LOG_LEVEL"])
        except KeyError:
            _current_log_level = 0
    else:
        _current_log_level = current_log_level

    if log_level <= _current_log_level:
        print(message, end='')
        stdout.flush()


# ----------------------------------------------------------------------
def colorize(text, color="yellow", activate=True):
    """
    This function return the text in a indicated color, if color is activate.

    :param text: Text to colorize
    :type text: basestring

    :param color: Color name
    :type color: basestring

    :param activate: boolean value that indicates if color is enabled.
    :type activate: bool

    :return: string with colored (if activated) text.
    :rtype: str
    """
    if activate:
        return colored(text, color)


# ----------------------------------------------------------------------
def generate_error_page(url):
    """
    Takes an URL to an existing document and generates a random URL
    to a nonexisting document, to trigger a server error.

    Example:

    >>> from libs.utils import generate_error_page
    >>> generate_error_page("http://www.site.com/index.php")
    'http://www.site.com/index.php.19ds_8vjX'

    :param url: Original URL.
    :type  url: str

    :return: Generated URL.
    :rtype: str
    """
    if not isinstance(url, str):
        raise TypeError("Expected basestring, got '%s' instead" % type(url))

    # Get random path
    random_path = "".join(choice(ascii_letters + digits) for _ in range(randint(5, 20)))

    return "%s/%s" % (url, random_path)


# ------------------------------------------------------------------------------
def get_diff_ratio(text1, text2):
    """
    Compare two texts and return a floating point value between 0 and 1 with
    the difference ratio, with 0 being absolutely different and 1 being
    absolutely equal - the more similar the two texts are, the closer the ratio
    will be to 1.

    ..note:
        This function was taken from Golismero project: http://github.com/golismero/golismero

    :param text1: First text to compare.
    :type text1: basestring

    :param text2: Second text to compare.
    :type text2: basestring

    :returns: Floating point value between 0 and 1.
    :rtype: float
    """

    # Solve some trivial type errors (like using None).
    if not text1:
        text1 = ""
    if not text2:
        text2 = ""

    # Trivial case, the two texts are identical.
    if text1 == text2:
        return 1.0

    # Use the difflib sequence matcher to calculate the ratio.
    m = SequenceMatcher(a=text1, b=text2)
    return m.ratio()


# ----------------------------------------------------------------------
def get_data_folder():
    """
    Return path of resources folder.

    :return: path of resources folder.
    :rtype: str
    """
    return abspath(join(dirname(__file__), "..", "resources"))


# ----------------------------------------------------------------------
def update_progress(values, print_function=None, prefix_text="", bar_len=40):
    """
    Creates a process bar in ASCII


    >>> for x in update_progress(range(1, 65), prefix_text="Prefix text: "):
    Prefix text: [########] 20.31%

    :param values: list or generator with items to process
    :type values: list

    :param print_function: function used to display information. By default is buildin 'print'.
    :type print_function: function

    :param prefix_text: Text to write before bar.
    :type prefix_text: str

    :param bar_len: number of characters used into bar.
    :type bar_len: int

    """
    _values = list(values)
    _len_values = len(_values)
    _var_len = bar_len
    _print = print_function or print

    for i, x in enumerate(_values, start=1):
        _percent = (i/_len_values)
        _percent_fix = int((i/_len_values) * _var_len)
        _print('\r{0} [#{1}] {2:.2f}%'.format(prefix_text, '#'*_percent_fix, _percent*100), end='')
        yield x


# ------------------------------------------------------------------------------
@asyncio.coroutine
def download(url, max_tries=3, max_redirect=2, connector=None, loop=None, method="get", get_content=True,
             auto_redirect=True):
    """
    Download a web page content.

    :param url: path where to get information.
    :type url: basestring

    :param max_tries: maximum number of retries for each page
    :type max_tries: int

    :param connector: HTTPConnection instance
    :type connector: `HTTPConnection`

    :param loop: Event loop object
    :type loop: loop

    :param method: HTTP method to use
    :type method: str

    :param get_content: boolean value that indicates if must download content or not
    :type get_content: bool

    :return: Web page content as a tuple: (http_header, status, basestring)
    :rtype: (dict, int, str)
    """
    _loop = loop or asyncio.get_event_loop()

    if max_redirect < 0:
        return None, None, None

    tries = 0

    while tries < max_tries:
        try:
            response = yield from aiohttp.request(
                # 'get',
                method,
                url,
                connector=connector,
                allow_redirects=False,
                loop=_loop)
            if tries > 1:
                log('\n[!] try %r for %r success\n', tries, url)
            break
        except aiohttp.ClientError as client_error:
            log("\n[!] Can't get before %r for %r tries, raised %r\n" % (
                colorize(tries, "red"), colorize(url, "red"), client_error),
                log_level=3)

        tries += 1
    else:
        # We never broke out of the loop: all tries failed.
        log('\n[!] %r failed after %r tries\n' % (url, max_tries))
        return

    if response.status in (300, 301, 302, 303, 307):
        location = response.headers.get('location')
        next_url = urllib.parse.urljoin(url, location)
        if max_redirect > 0:
            log('\n[!] redirect to %r from %r\n' % (next_url, url), log_level=1)
            if auto_redirect is True:
                return _loop.run_until_complete(download(next_url,
                                                         max_redirect=(max_redirect-1)))
            else:
                return response.headers, response.status, None
        else:
            log('\n[!] redirect limit reached for %r from %r\n' % (next_url, url), log_level=2)
            return response.headers, response.status, None
    else:
        content = None

        if get_content:
            content = (yield from response.read()).decode(errors="ignore")

        return response.headers, response.status, content


# --------------------------------------------------------------------------
class ConcurrentDownloader:
    """Get a list of URLs. Download their content and call user defined action"""

    # ----------------------------------------------------------------------
    def __init__(self, process_url_content, max_redirects=2, max_tries=4, max_tasks=10, loop=None,
                 connector=None):
        """
        :param process_url_content: function to process URL content, after it is downloaded
        :type process_url_content: function

        :param max_redirects: maximum number of redirects
        :type max_redirects: int

        :param max_tries: maximum number of HTTP retries.
        :type max_tries: int

        :param max_tasks: maximum number of concurrent tasks
        :type max_tasks: int

        :param loop: optional event loop object
        :type loop: loop

        :param connector: aioTCPConnector object
        :type connector: aiohttp.TCPConnector


        >>> import asyncio
        >>> display=lambda x: print(x)
        >>> loop = asyncio.get_event_loop()
        >>> v = ConcurrentDownloader(url_base="http://myhost.com", process_url_content=display)
        >>> loop.run_until_complete(v.run())
        """
        self.max_redirects = max_redirects
        self.process_url_function = process_url_content or (lambda x: None)
        self.max_tries = max_tries
        self.max_tasks = max_tasks
        self.loop = loop or asyncio.get_event_loop()
        self.q = asyncio.JoinableQueue(loop=self.loop)
        self.connector = connector or aiohttp.TCPConnector(loop=self.loop)
        self.__results = []
        self.__results_append = self.results.append

    # ----------------------------------------------------------------------
    @property
    def results(self):
        return self.__results

    # ----------------------------------------------------------------------
    @asyncio.coroutine
    def _work(self):
        while True:
            # Get an URL to process
            url = yield from self.q.get()

            # Download content
            log("\n    |- Trying: %s" % colorize(url, "yellow"), log_level=1)

            headers, status, content = yield from download(url,
                                                           self.max_tries,
                                                           self.max_redirects,
                                                           self.connector,
                                                           self.loop)

            # Processing response
            _r = self.process_url_function(url, headers, status, content)
            if _r is not None:
                self.__results_append(_r)

            del headers, status, content

            self.q.task_done()

    # --------------------------------------------------------------------------
    #
    # Public method
    #
    # ----------------------------------------------------------------------
    def close(self):
        """Close resources."""
        self.connector.close()

    # --------------------------------------------------------------------------
    @asyncio.coroutine
    def run(self):
        """
        Start the analyzer daemon.
        """

        # Add workers
        workers = [asyncio.Task(self._work(), loop=self.loop)
                   for _ in range(self.max_tasks)]

        # Wait content of workers ends
        yield from self.q.join()

        for w in workers:
            w.cancel()

    # ----------------------------------------------------------------------
    def add_url(self, url):
        """
        Add a URL to queue to analyze

        :param url: URL to store
        :type url: str
        """
        self.q.put_nowait(url)

    # ----------------------------------------------------------------------
    def add_url_list(self, urls):
        """
        Add an URL list to processing
        
        :param urls: list with URLs 
        :type urls: str
        
        """
        for x in urls:
            self.q.put_nowait(x)


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Plecost: Wordpress vulnerabilities finder
#
# @url: http://iniqua.com/labs/
# @url: https://github.com/iniqua/plecost
#
# @author:Francisco J. Gomez aka ffranz (http://iniqua.com/)
# @author:Daniel Garcia aka cr0hn (http://www.cr0hn.com/me/)
#
# Copyright (c) 2015, Iniqua Team
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
# following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
# following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
# products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


import sqlite3

from os.path import exists, join

from .utils import get_data_folder
from .data import PlecostDatabaseQuery


# --------------------------------------------------------------------------
class DB:
    """DB handler"""

    # ----------------------------------------------------------------------
    def __init__(self, path, auto_create=True):
        """
        :param path: path to sqlite database
        :type path: str

        :param auto_create: auto create db if not found
        :type auto_create: bool

        :param overwrite: if database exits, overwrite them
        :type overwrite: bool

        """
        self.path = path

        # We need to create initial tables?
        if not exists(path):
            if auto_create:
                self.con = self._create_db(self.path)
            else:
                raise IOError("Database '%r' not found." % path)
        else:
            self.con = sqlite3.connect(path)

    # ----------------------------------------------------------------------
    def _get_rows(self, rows):
        _rows = rows.fetchall()
        if _rows is not None:
            return [x[0] for x in _rows]
        else:
            return None

    # ----------------------------------------------------------------------
    def _get_single_row(self, rows):
        _rows = rows.fetchone()
        if _rows is not None:
            return _rows[0]
        else:
            return None

    # ----------------------------------------------------------------------
    def query_plugin(self, plugin_name, plugin_version=None):
        """
        Query for plugin version and return associated CVE, if exists.

        :param plugin_name: plugin name
        :type plugin_name: str

        :param plugin_version: plugin version
        :type plugin_version: str

        :return: CVE value
        :rtype: str|None
        """
        if plugin_version is not None:
            _plugin_version = "%"
        else:
            _plugin_version = plugin_version

        r = self.con.execute("SELECT PVC.cve "
                             "FROM PLUGIN_VULNERABILITIES as PV, PLUGIN_VULNERABILITIES_CVE as PVC "
                             "WHERE PV._id == PVC._id AND PV.plugin_name LIKE ? AND PV.plugin_version LIKE ?",
                             (plugin_name, plugin_version,))

        return self._get_rows(r)

    # ----------------------------------------------------------------------
    def query_wordpress(self, wordpress_version):
        """
        Query for wordpress version and return associated CVEs, if exists.

        :param wordpress_version: wordpress version
        :type wordpress_version: str

        :return: CVE value
        :rtype: str|None
        """
        r = self.con.execute("SELECT WVC.cve "
                             "FROM WORDPRESS_VULNERABILITIES as WV, WORDPRESS_VULNERABILITIES_CVE as WVC "
                             "WHERE WV.wordpress_version == WVC.wordpress_version AND WV.wordpress_version LIKE ?",
                             (wordpress_version,))

        return self._get_rows(r)

    # ----------------------------------------------------------------------
    def query_cve(self, cve):
        """
        Get CVE description

        :return: CVE description
        :rtype: str

        """
        r = self.con.execute("SELECT cve_description FROM CVE WHERE CVE.cve = ?", (cve, ))

        return self._get_single_row(r)

    # ----------------------------------------------------------------------
    def clean_db(self):

        # Remove existing info?
        tables = ["PLUGIN_VULNERABILITIES",
                  "PLUGIN_VULNERABILITIES_CVE",
                  "WORDPRESS_VULNERABILITIES",
                  "WORDPRESS_VULNERABILITIES_CVE",
                  "CVE"]

        for table in tables:
            self.con.execute("DROP TABLE IF EXISTS %s;" % table)

    # ----------------------------------------------------------------------
    def raw(self, query, parameters=()):
        """
        Make raw query
        """
        return self.con.execute(query, parameters)

    # ----------------------------------------------------------------------
    def create_db(self):
        """
        Creates databases used for storing information

        :return: sqlite connection
        :rtype: sqlite.Connection

        """
        tables = dict(
            q_table_vulns="CREATE TABLE PLUGIN_VULNERABILITIES ("
                          "_id INTEGER PRIMARY KEY autoincrement,"
                          "plugin_name TEXT NOT NULL, "
                          "plugin_version TEXT NOT NULL,"
                          "UNIQUE (plugin_name, plugin_version));",

            q_table_vulns_cve="CREATE TABLE PLUGIN_VULNERABILITIES_CVE ("
                              "_id TEXT REFERENCES PLUGIN_VULNERABILITIES(_id), "
                              "cve TEXT REFERENCES cve(cve));",

            q_table_wordpress="CREATE TABLE WORDPRESS_VULNERABILITIES (wordpress_version TEXT PRIMARY KEY NOT NULL);",

            q_table_wordpress_cve="CREATE TABLE WORDPRESS_VULNERABILITIES_CVE ("
                                  "wordpress_version TEXT REFERENCES WORDPRESS_VULNERABILITIES(wordpress_version), "
                                  "CVE TEXT REFERENCES cve(cve));",

            q_table_cve="CREATE TABLE CVE ("
                        "cve TEXT PRIMARY KEY NOT NULL, "
                        "cve_description TEXT, "
                        "author_page TEXT);")

        con = sqlite3.connect(self.path)

        # Create tables
        for query in tables.values():
            con.execute(query)

        con.commit()

        self.con = con

        return con


# --------------------------------------------------------------------------
#
# Database query operations
#
# --------------------------------------------------------------------------

# ----------------------------------------------------------------------
def __cve_details(data, db):
    """
    Get a CVE details.
    """
    _cve = data.parameter
    _query = "SELECT cve_description FROM CVE WHERE cve LIKE ?;"

    res = []
    res_append = res.append

    # Title
    res_append("[*] Detail for CVE '%s':" % _cve)

    r = db.raw(query=_query, parameters=(_cve,)).fetchone()

    if r is not None:
        res_append("\n    %s" % r[0])

    res_append("\n")

    return "\n".join(res)


# ----------------------------------------------------------------------
def __plugin_cves(data, db):
    """
    Get CVEs of a plugin

    :param data: PlecostDatabaseQuery object
    :type data: PlecostDatabaseQuery

    :return: results of query
    :rtype: str
    """
    _plugin = data.parameter

    _query = ("SELECT DISTINCT(cve) "
              "FROM PLUGIN_VULNERABILITIES_CVE as PV, (SELECT DISTINCT(plugin_name), "
              "_id FROM PLUGIN_VULNERABILITIES WHERE plugin_name LIKE ?) as PN "
              "WHERE PN._id == PV._id")

    _query_versions = ("SELECT DISTINCT(plugin_version) "
                       "FROM PLUGIN_VULNERABILITIES AS PV, PLUGIN_VULNERABILITIES_CVE AS PC "
                       "WHERE PV._id == PC._id AND PC.cve like ?;")

    res = []
    res_append = res.append

    # Title
    res_append("[*] Associated CVEs for plugin '%s':\n" % _plugin)

    for i, x in enumerate(db.raw(query=_query, parameters=(_plugin,)).fetchall()):
        _cve = x[0]
        res_append("    { %s } - %s:\n" % (i, _cve))

        # Get associated versions
        res_append("             Affected versions:\n")
        for j, v in enumerate(db.raw(_query_versions, parameters=(_cve, )).fetchall()):
            _version = v[0]
            res_append("             <%s> - %s" % (j, _version))

    res_append("\n")

    return "\n".join(res)


# ----------------------------------------------------------------------
def __plugin_list(data, db):
    """
    Get plugin list in vulnerability database

    :param data: PlecostDatabaseQuery object
    :type data: PlecostDatabaseQuery

    :return: results of query
    :rtype: str
    """
    res = []
    res_append = res.append

    # Title
    res_append("[*] Plugins with vulnerabilities known:\n")

    for i, x in enumerate(db.raw("SELECT DISTINCT(plugin_name) FROM PLUGIN_VULNERABILITIES;").fetchall()):
        res_append("    { %s } - %s" % (i, x[0]))

    res_append("\n")

    return "\n".join(res)


# ----------------------------------------------------------------------
def db_query(data):
    """
    Query the database and return a text with the information.

    :param data: PlecostDatabaseQuery object
    :type data: PlecostDatabaseQuery

    :return: results of query
    :rtype: str

    """
    if not isinstance(data, PlecostDatabaseQuery):
        raise TypeError("Expected PlecostDatabaseQuery, got '%s' instead" % type(data))

    _actions = dict(plugin_list=__plugin_list,
                    cve=__cve_details,
                    plugin_cves=__plugin_cves)

    db = DB(join(get_data_folder(), "cve.db"))

    return _actions[data.action](data, db)